@@ -154,7 +154,7 @@ class interp2d(object):
     fill_value : number, optional
         If provided, the value to use for points outside of the
         interpolation domain. If omitted (None), values outside
-        the domain are extrapolated.
+        the domain are extrapolated via nearest-neighbor extrapolation.
 
     See Also
     --------
@@ -1211,7 +1211,7 @@ class PPoly(_PPolyBase):
 
         >>> from scipy.interpolate import PPoly
         >>> pp = PPoly(np.array([[1, -4, 3], [1, 0, 0]]).T, [-2, 1, 2])
-        >>> pp.roots()
+        >>> pp.solve()
         array([-1.,  1.])
         """
         if extrapolate is None:
@@ -1310,6 +1310,10 @@ class PPoly(_PPolyBase):
             based on first and last intervals, or to return NaNs.
             If 'periodic', periodic extrapolation is used. Default is True.
         """
+        if not isinstance(bp, BPoly):
+            raise TypeError(".from_bernstein_basis only accepts BPoly instances. "
+                            "Got %s instead." % type(bp))
+
         dx = np.diff(bp.x)
         k = bp.c.shape[0] - 1  # polynomial order
 
@@ -1614,6 +1618,10 @@ class BPoly(_PPolyBase):
             based on first and last intervals, or to return NaNs.
             If 'periodic', periodic extrapolation is used. Default is True.
         """
+        if not isinstance(pp, PPoly):
+            raise TypeError(".from_power_basis only accepts PPoly instances. "
+                            "Got %s instead." % type(pp))
+
         dx = np.diff(pp.x)
         k = pp.c.shape[0] - 1   # polynomial order
 
