@@ -5,8 +5,9 @@ from the files "cython_blas_signatures.txt" and
 all the BLAS/LAPACK routines that should be included in the wrappers.
 """
 
-import os
+from collections import defaultdict
 from operator import itemgetter
+import os
 
 BASE_DIR = os.path.abspath(os.path.dirname(__file__))
 
@@ -166,8 +167,8 @@ Usable from Cython via::
     cimport scipy.linalg.cython_lapack
 
 This module provides Cython-level wrappers for all primary routines included
-in LAPACK 3.1.0 except for ``zcgesv`` since its interface is not consistent
-from LAPACK 3.1.0 to 3.6.0. It also provides some of the
+in LAPACK 3.4.0 except for ``zcgesv`` since its interface is not consistent
+from LAPACK 3.4.0 to 3.6.0. It also provides some of the
 fixed-api auxiliary routines.
 
 These wrappers do not check for alignment of arrays.
@@ -527,37 +528,55 @@ def generate_lapack_pxd(all_sigs):
     return lapack_pxd_preamble + '\n'.join(pxd_decl(*sig) for sig in all_sigs)
 
 
-fortran_template = """      subroutine {name}wrp(ret, {argnames})
+fortran_template = """      subroutine {name}wrp(
+     +    ret,
+     +    {argnames}
+     +    )
         external {wrapper}
         {ret_type} {wrapper}
         {ret_type} ret
         {argdecls}
-        ret = {wrapper}({argnames})
+        ret = {wrapper}(
+     +    {argnames}
+     +    )
       end
 """
 
 dims = {'work': '(*)', 'ab': '(ldab,*)', 'a': '(lda,*)', 'dl': '(*)',
         'd': '(*)', 'du': '(*)', 'ap': '(*)', 'e': '(*)', 'lld': '(*)'}
 
+xy_specialized_dims = {'x': '', 'y': ''}
+a_specialized_dims = {'a': '(*)'}
+special_cases = defaultdict(dict,
+                            ladiv = xy_specialized_dims,
+                            lanhf = a_specialized_dims,
+                            lansf = a_specialized_dims,
+                            lapy2 = xy_specialized_dims,
+                            lapy3 = xy_specialized_dims)
+
 
 def process_fortran_name(name, funcname):
     if 'inc' in name:
         return name
-    xy_exclusions = ['ladiv', 'lapy2', 'lapy3']
-    if ('x' in name or 'y' in name) and funcname[1:] not in xy_exclusions:
-        return name + '(n)'
-    if name in dims:
-        return name + dims[name]
+    special = special_cases[funcname[1:]]
+    if 'x' in name or 'y' in name:
+        suffix = special.get(name, '(n)')
+    else:
+        suffix = special.get(name, '')
+    return name + suffix
+
+
+def called_name(name):
+    included = ['cdotc', 'cdotu', 'zdotc', 'zdotu', 'cladiv', 'zladiv']
+    if name in included:
+        return "w" + name
     return name
 
 
 def fort_subroutine_wrapper(name, ret_type, args):
-    if name[0] in ['c', 's'] or name in ['zladiv', 'zdotu', 'zdotc']:
-        wrapper = 'w' + name
-    else:
-        wrapper = name
+    wrapper = called_name(name)
     types, names = arg_names_and_types(args)
-    argnames = ', '.join(names)
+    argnames = ',\n     +    '.join(names)
 
     names = [process_fortran_name(n, name) for n in names]
     argdecls = '\n        '.join('{0} {1}'.format(fortran_types[t], n)
@@ -577,7 +596,8 @@ def make_c_args(args):
     return ', '.join('{0} *{1}'.format(t, n) for t, n in zip(types, names))
 
 
-c_func_template = "void F_FUNC({name}wrp, {upname}WRP)({return_type} *ret, {args});\n"
+c_func_template = ("void F_FUNC({name}wrp, {upname}WRP)"
+                   "({return_type} *ret, {args});\n")
 
 
 def c_func_decl(name, return_type, args):
@@ -644,8 +664,8 @@ def split_signature(sig):
 
 
 def filter_lines(lines):
-    lines = [line.strip() for line in lines
-                              if line != '\n' and line[0] != '#']
+    lines = [line for line in map(str.strip, lines)
+                      if line and not line.startswith('#')]
     func_sigs = [split_signature(line) for line in lines
                                            if line.split(' ')[0] != 'void']
     sub_sigs = [split_signature(line) for line in lines
@@ -689,7 +709,8 @@ def make_all(blas_signature_file="cython_blas_signatures.txt",
 
     comments = ["This file was generated by _generate_pyx.py.\n",
                 "Do not edit this file directly.\n"]
-    ccomment = ''.join(['/* ' + line.rstrip() + ' */\n' for line in comments]) + '\n'
+    ccomment = ''.join(['/* ' + line.rstrip() + ' */\n'
+                        for line in comments]) + '\n'
     pyxcomment = ''.join(['# ' + line for line in comments]) + '\n'
     fcomment = ''.join(['c     ' + line for line in comments]) + '\n'
     with open(blas_signature_file, 'r') as f:
