@@ -39,33 +39,49 @@ robert.kern@gmail.com
 from __future__ import division, print_function, absolute_import
 
 import numpy
+from warnings import warn
 from scipy.odr import __odrpack
 
-__all__ = ['odr', 'odr_error', 'odr_stop', 'Data', 'RealData', 'Model',
-           'Output', 'ODR']
+__all__ = ['odr', 'OdrWarning', 'OdrError', 'OdrStop',
+           'Data', 'RealData', 'Model', 'Output', 'ODR',
+           'odr_error', 'odr_stop']
 
 odr = __odrpack.odr
 
 
-class odr_error(Exception):
+class OdrWarning(UserWarning):
     """
-    Exception indicating an error in fitting.
-
-    This is raised by `scipy.odr` if an error occurs during fitting.
+    Warning indicating that the data passed into
+    ODR will cause problems when passed into 'odr'
+    that the user should be aware of.
     """
     pass
 
 
-class odr_stop(Exception):
+class OdrError(Exception):
+    """
+    Exception indicating an error in fitting.
+
+    This is raised by `~scipy.odr.odr` if an error occurs during fitting.
+    """
+    pass
+
+
+class OdrStop(Exception):
     """
     Exception stopping fitting.
 
     You can raise this exception in your objective function to tell
-    `scipy.odr` to stop fitting.
+    `~scipy.odr.odr` to stop fitting.
     """
     pass
 
-__odrpack._set_exceptions(odr_error, odr_stop)
+
+# Backwards compatibility
+odr_error = OdrError
+odr_stop = OdrStop
+
+__odrpack._set_exceptions(OdrError, OdrStop)
 
 
 def _conv(obj, dtype=None):
@@ -171,9 +187,11 @@ class Data(object):
     Parameters
     ----------
     x : array_like
-        Input data for regression.
+        Observed data for the independent variable of the regression
     y : array_like, optional
-        Input data for regression.
+        If array-like, observed data for the dependent variable of the
+        regression. A scalar input implies that the model to be used on
+        the data is implicit.
     we : array_like, optional
         If `we` is a scalar, then that value is used for all data points (and
         all dimensions of the response variable).
@@ -242,6 +260,12 @@ class Data(object):
 
     def __init__(self, x, y=None, we=None, wd=None, fix=None, meta={}):
         self.x = _conv(x)
+
+        if not isinstance(self.x, numpy.ndarray):
+            raise ValueError(("Expected an 'ndarray' of data for 'x', "
+                              "but instead got data of type '{name}'").format(
+                    name=type(self.x).__name__))
+
         self.y = _conv(y)
         self.we = _conv(we)
         self.wd = _conv(wd)
@@ -278,10 +302,12 @@ class RealData(Data):
     Parameters
     ----------
     x : array_like
-        x
+        Observed data for the independent variable of the regression
     y : array_like, optional
-        y
-    sx, sy : array_like, optional
+        If array-like, observed data for the dependent variable of the
+        regression. A scalar input implies that the model to be used on
+        the data is implicit.
+    sx : array_like, optional
         Standard deviations of `x`.
         `sx` are standard deviations of `x` and are converted to weights by
         dividing 1.0 by their squares.
@@ -348,6 +374,12 @@ class RealData(Data):
             self._ga_flags['we'] = 'covy'
 
         self.x = _conv(x)
+
+        if not isinstance(self.x, numpy.ndarray):
+            raise ValueError(("Expected an 'ndarray' of data for 'x', "
+                              "but instead got data of type '{name}'").format(
+                    name=type(self.x).__name__))
+
         self.y = _conv(y)
         self.sx = _conv(sx)
         self.sy = _conv(sy)
@@ -450,8 +482,8 @@ class Model(object):
         If the input data is multi-dimensional, then `x` is a rank-2 array;
         i.e., ``x = array([[1, 2, ...], [2, 4, ...]]); x.shape = (m, n)``.
         In all cases, it has the same shape as the input data array passed to
-        `odr`. `m` is the dimensionality of the input data, `n` is the number
-        of observations.
+        `~scipy.odr.odr`. `m` is the dimensionality of the input data,
+        `n` is the number of observations.
     `y`
         if the response variable is single-dimensional, then `y` is a
         rank-1 array, i.e., ``y = array([2, 4, ...]); y.shape = (n,)``.
@@ -532,7 +564,7 @@ class Output(object):
         Array ``y = fcn(x + delta)``.
     res_var : float, optional
         Residual variance.
-    sum_sqare : float, optional
+    sum_square : float, optional
         Sum of squares error.
     sum_square_delta : float, optional
         Sum of squares of delta error.
@@ -554,8 +586,8 @@ class Output(object):
     Notes
     -----
     Takes one argument for initialization, the return value from the
-    function `odr`. The attributes listed as "optional" above are only
-    present if `odr` was run with ``full_output=1``.
+    function `~scipy.odr.odr`. The attributes listed as "optional" above are
+    only present if `~scipy.odr.odr` was run with ``full_output=1``.
 
     """
 
@@ -710,6 +742,9 @@ class ODR(object):
         else:
             self.beta0 = _conv(beta0)
 
+        if ifixx is None and data.fix is not None:
+            ifixx = data.fix
+
         self.delta0 = _conv(delta0)
         # These really are 32-bit integers in FORTRAN (gfortran), even on 64-bit
         # platforms.
@@ -746,16 +781,16 @@ class ODR(object):
         if isinstance(self.data.y, numpy.ndarray):
             y_s = list(self.data.y.shape)
             if self.model.implicit:
-                raise odr_error("an implicit model cannot use response data")
+                raise OdrError("an implicit model cannot use response data")
         else:
             # implicit model with q == self.data.y
             y_s = [self.data.y, x_s[-1]]
             if not self.model.implicit:
-                raise odr_error("an explicit model needs response data")
+                raise OdrError("an explicit model needs response data")
             self.set_job(fit_type=1)
 
         if x_s[-1] != y_s[-1]:
-            raise odr_error("number of observations do not match")
+            raise OdrError("number of observations do not match")
 
         n = x_s[-1]
 
@@ -800,24 +835,29 @@ class ODR(object):
         if res.shape not in fcn_perms:
             print(res.shape)
             print(fcn_perms)
-            raise odr_error("fcn does not output %s-shaped array" % y_s)
+            raise OdrError("fcn does not output %s-shaped array" % y_s)
 
         if self.model.fjacd is not None:
             res = self.model.fjacd(*arglist)
             if res.shape not in fjacd_perms:
-                raise odr_error(
-                    "fjacd does not output %s-shaped array" % (q, m, n))
+                raise OdrError(
+                    "fjacd does not output %s-shaped array" % repr((q, m, n)))
         if self.model.fjacb is not None:
             res = self.model.fjacb(*arglist)
             if res.shape not in fjacb_perms:
-                raise odr_error(
-                    "fjacb does not output %s-shaped array" % (q, p, n))
+                raise OdrError(
+                    "fjacb does not output %s-shaped array" % repr((q, p, n)))
 
         # check shape of delta0
 
         if self.delta0 is not None and self.delta0.shape != self.data.x.shape:
-            raise odr_error(
-                "delta0 is not a %s-shaped array" % self.data.x.shape)
+            raise OdrError(
+                "delta0 is not a %s-shaped array" % repr(self.data.x.shape))
+
+        if self.data.x.size == 0:
+            warn(("Empty data detected for ODR instance. "
+                  "Do not expect any fitting to occur"),
+                 OdrWarning)
 
     def _gen_work(self):
         """ Generate a suitable work array if one does not already exist.
@@ -991,7 +1031,7 @@ class ODR(object):
             (so_init is not None or
              so_iter is not None or
              so_final is not None)):
-            raise odr_error(
+            raise OdrError(
                 "no rptfile specified, cannot output to stdout twice")
 
         iprint_l = ip2arg[ip[0]] + ip2arg[ip[1]] + ip2arg[ip[3]]
@@ -1020,7 +1060,7 @@ class ODR(object):
         self.iprint = ip[0]*1000 + ip[1]*100 + ip[2]*10 + ip[3]
 
     def run(self):
-        """ Run the fitting routine with all of the information given.
+        """ Run the fitting routine with all of the information given and with ``full_output=1``.
 
         Returns
         -------
@@ -1079,7 +1119,7 @@ class ODR(object):
         """
 
         if self.output is None:
-            raise odr_error("cannot restart: run() has not been called before")
+            raise OdrError("cannot restart: run() has not been called before")
 
         self.set_job(restart=1)
         self.work = self.output.work
