@@ -1,6 +1,6 @@
 """Revised simplex method for linear programming
 
-The *revised simplex* method uses the method decribed in [1]_, except
+The *revised simplex* method uses the method described in [1]_, except
 that a factorization [2]_ of the basis matrix, rather than its inverse,
 is efficiently maintained and used to solve the linear systems at each
 iteration of the algorithm.
@@ -29,8 +29,8 @@ from ._linprog_util import _postsolve
 from .optimize import OptimizeResult
 
 
-def _phase_one(A, b, x0, maxiter, tol, maxupdate, mast, pivot, callback=None,
-               _T_o=[], disp=False):
+def _phase_one(A, b, x0, callback, postsolve_args, maxiter, tol, disp,
+               maxupdate, mast, pivot):
     """
     The purpose of phase one is to find an initial basic feasible solution
     (BFS) to the original problem.
@@ -57,9 +57,12 @@ def _phase_one(A, b, x0, maxiter, tol, maxupdate, mast, pivot, callback=None,
 
     # solve auxiliary problem
     phase_one_n = n
-    x, basis, status, iter_k = _phase_two(c, A, x, basis, maxiter,
-                                          tol, maxupdate, mast, pivot,
-                                          0, callback, _T_o, disp, phase_one_n)
+    iter_k = 0
+    x, basis, status, iter_k = _phase_two(c, A, x, basis, callback,
+                                          postsolve_args,
+                                          maxiter, tol, disp,
+                                          maxupdate, mast, pivot,
+                                          iter_k, phase_one_n)
 
     # check for infeasibility
     residual = c.dot(x)
@@ -134,7 +137,7 @@ def _get_more_basis_columns(A, basis):
 def _generate_auxiliary_problem(A, b, x0, tol):
     """
     Modifies original problem to create an auxiliary problem with a trivial
-    intial basic feasible solution and an objective that minimizes
+    initial basic feasible solution and an objective that minimizes
     infeasibility in the original problem.
 
     Conceptually this is done by stacking an identity matrix on the right of
@@ -307,8 +310,8 @@ def _display_iter(phase, iteration, slack, con, fun):
     print(fmt.format(phase, iteration, slack, np.linalg.norm(con), fun))
 
 
-def _phase_two(c, A, x, b, maxiter, tol, maxupdate, mast, pivot, iteration=0,
-               callback=None, _T_o=[], disp=False, phase_one_n=None):
+def _phase_two(c, A, x, b, callback, postsolve_args, maxiter, tol, disp,
+               maxupdate, mast, pivot, iteration=0, phase_one_n=None):
     """
     The heart of the simplex method. Beginning with a basic feasible solution,
     moves to adjacent basic feasible solutions successively lower reduced cost.
@@ -340,7 +343,8 @@ def _phase_two(c, A, x, b, maxiter, tol, maxupdate, mast, pivot, iteration=0,
             else:
                 phase = 2
                 x_postsolve = x
-            x_o, fun, slack, con, _, _ = _postsolve(x_postsolve, *_T_o,
+            x_o, fun, slack, con, _, _ = _postsolve(x_postsolve,
+                                                    postsolve_args,
                                                     tol=tol, copy=True)
 
             if callback is not None:
@@ -398,8 +402,9 @@ def _phase_two(c, A, x, b, maxiter, tol, maxupdate, mast, pivot, iteration=0,
     return x, b, status, iteration
 
 
-def _linprog_rs(c, c0, A, b, x0=None, callback=None, maxiter=5000, tol=1e-12,
-                maxupdate=10, mast=False, pivot="mrc", _T_o=[], disp=False,
+def _linprog_rs(c, c0, A, b, x0, callback, postsolve_args,
+                maxiter=5000, tol=1e-12, disp=False,
+                maxupdate=10, mast=False, pivot="mrc",
                 **unknown_options):
     """
     Solve the following linear programming problem via a two-phase
@@ -456,6 +461,9 @@ def _linprog_rs(c, c0, A, b, x0=None, callback=None, maxiter=5000, tol=1e-12,
                 The number of iterations performed.
             message : str
                 A string descriptor of the exit status of the optimization.
+    postsolve_args : tuple
+        Data needed by _postsolve to convert the solution to the standard-form
+        problem into the solution to the original problem.
 
     Options
     -------
@@ -465,6 +473,9 @@ def _linprog_rs(c, c0, A, b, x0=None, callback=None, maxiter=5000, tol=1e-12,
         The tolerance which determines when a solution is "close enough" to
         zero in Phase 1 to be considered a basic feasible solution or close
         enough to positive to serve as an optimal solution.
+    disp : bool
+        Set to ``True`` if indicators of optimization status are to be printed
+        to the console each iteration.
     maxupdate : int
         The maximum number of updates performed on the LU factorization.
         After this many updates is reached, the basis matrix is factorized
@@ -482,9 +493,10 @@ def _linprog_rs(c, c0, A, b, x0=None, callback=None, maxiter=5000, tol=1e-12,
     pivot : "mrc" or "bland"
         Pivot rule: Minimum Reduced Cost (default) or Bland's rule. Choose
         Bland's rule if iteration limit is reached and cycling is suspected.
-    disp : bool
-        Set to ``True`` if indicators of optimization status are to be printed
-        to the console each iteration.
+    unknown_options : dict
+        Optional arguments not used by this particular solver. If
+        `unknown_options` is non-empty a warning is issued listing all
+        unused options.
 
     Returns
     -------
@@ -529,23 +541,18 @@ def _linprog_rs(c, c0, A, b, x0=None, callback=None, maxiter=5000, tol=1e-12,
                 "solution. "
                 ]
 
-    # _T_o contains information for postsolve needed for callback function
-    # callback function also needs `complete` argument
-    # I add `complete = False` here for convenience
-    _T_o = list(_T_o)
-    _T_o.insert(-1, False)
-
     if A.size == 0:  # address test_unbounded_below_no_presolve_corrected
         return np.zeros(c.shape), 5, messages[5], 0
 
     x, basis, A, b, residual, status, iteration = (
-        _phase_one(A, b, x0, maxiter, tol, maxupdate,
-                   mast, pivot, callback, _T_o, disp))
+        _phase_one(A, b, x0, callback, postsolve_args,
+                   maxiter, tol, disp, maxupdate, mast, pivot))
 
     if status == 0:
-        x, basis, status, iteration = _phase_two(c, A, x, basis,
-                                                 maxiter, tol, maxupdate,
-                                                 mast, pivot, iteration,
-                                                 callback, _T_o, disp)
+        x, basis, status, iteration = _phase_two(c, A, x, basis, callback,
+                                                 postsolve_args,
+                                                 maxiter, tol, disp,
+                                                 maxupdate, mast, pivot,
+                                                 iteration)
 
     return x, status, messages[status].format(residual, tol), iteration
