@@ -22,7 +22,7 @@ Sparse matrix classes
    csr_matrix - Compressed Sparse Row matrix
    dia_matrix - Sparse matrix with DIAgonal storage
    dok_matrix - Dictionary Of Keys based sparse matrix
-   lil_matrix - Row-based linked list sparse matrix
+   lil_matrix - Row-based list of lists sparse matrix
    spmatrix - Sparse matrix base class
 
 Functions
@@ -82,7 +82,6 @@ Submodules
 ----------
 
 .. autosummary::
-   :toctree: generated/
 
    csgraph - Compressed sparse graph routines
    linalg - sparse linear algebra routines
@@ -225,6 +224,8 @@ from __future__ import division, print_function, absolute_import
 # Modified and extended by Ed Schofield, Robert Cimrman,
 # Nathan Bell, and Jake Vanderplas.
 
+import warnings as _warnings
+
 from .base import *
 from .csr import *
 from .csc import *
@@ -237,11 +238,14 @@ from .construct import *
 from .extract import *
 from ._matrix_io import *
 
-# for backward compatibility with v0.10.  This function is marked as deprecated
-from .csgraph import cs_graph_components
-
-#from spfuncs import *
+# For backward compatibility with v0.19.
+from . import csgraph
 
 __all__ = [s for s in dir() if not s.startswith('_')]
-from numpy.testing import Tester
-test = Tester().test
+
+# Filter PendingDeprecationWarning for np.matrix introduced with numpy 1.15
+_warnings.filterwarnings('ignore', message='the matrix subclass is not the recommended way')
+
+from scipy._lib._testutils import PytestTester
+test = PytestTester(__name__)
+del PytestTester
