@@ -6,18 +6,15 @@ __docformat__ = "restructuredtext en"
 
 __all__ = ['dok_matrix', 'isspmatrix_dok']
 
-import functools
-import operator
 import itertools
-
 import numpy as np
 
 from scipy._lib.six import zip as izip, xrange, iteritems, iterkeys, itervalues
 
 from .base import spmatrix, isspmatrix
+from ._index import IndexMixin
 from .sputils import (isdense, getdtype, isshape, isintlike, isscalarlike,
-                      upcast, upcast_scalar, IndexMixin, get_index_dtype,
-                      check_shape)
+                      upcast, upcast_scalar, get_index_dtype, check_shape)
 
 try:
     from operator import isSequenceType as _is_sequence
@@ -101,7 +98,7 @@ class dok_matrix(spmatrix, IndexMixin, dict):
         else:  # Dense ctor
             try:
                 arg1 = np.asarray(arg1)
-            except:
+            except Exception:
                 raise TypeError('Invalid input format.')
 
             if len(arg1.shape) != 2:
@@ -160,152 +157,93 @@ class dok_matrix(spmatrix, IndexMixin, dict):
             raise IndexError('Index out of bounds.')
         return dict.get(self, key, default)
 
-    def __getitem__(self, index):
-        """If key=(i, j) is a pair of integers, return the corresponding
-        element.  If either i or j is a slice or sequence, return a new sparse
-        matrix with just these elements.
-        """
-        zero = self.dtype.type(0)
-        i, j = self._unpack_index(index)
+    def _get_intXint(self, row, col):
+        return dict.get(self, (row, col), self.dtype.type(0))
 
-        i_intlike = isintlike(i)
-        j_intlike = isintlike(j)
+    def _get_intXslice(self, row, col):
+        return self._get_sliceXslice(slice(row, row+1), col)
 
-        if i_intlike and j_intlike:
-            i = int(i)
-            j = int(j)
-            if i < 0:
-                i += self.shape[0]
-            if i < 0 or i >= self.shape[0]:
-                raise IndexError('Index out of bounds.')
-            if j < 0:
-                j += self.shape[1]
-            if j < 0 or j >= self.shape[1]:
-                raise IndexError('Index out of bounds.')
-            return dict.get(self, (i,j), zero)
-        elif ((i_intlike or isinstance(i, slice)) and
-              (j_intlike or isinstance(j, slice))):
-            # Fast path for slicing very sparse matrices
-            i_slice = slice(i, i+1) if i_intlike else i
-            j_slice = slice(j, j+1) if j_intlike else j
-            i_indices = i_slice.indices(self.shape[0])
-            j_indices = j_slice.indices(self.shape[1])
-            i_seq = xrange(*i_indices)
-            j_seq = xrange(*j_indices)
-            newshape = (len(i_seq), len(j_seq))
-            newsize = _prod(newshape)
+    def _get_sliceXint(self, row, col):
+        return self._get_sliceXslice(row, slice(col, col+1))
 
-            if len(self) < 2*newsize and newsize != 0:
-                # Switch to the fast path only when advantageous
-                # (count the iterations in the loops, adjust for complexity)
-                #
-                # We also don't handle newsize == 0 here (if
-                # i/j_intlike, it can mean index i or j was out of
-                # bounds)
-                return self._getitem_ranges(i_indices, j_indices, newshape)
+    def _get_sliceXslice(self, row, col):
+        row_start, row_stop, row_step = row.indices(self.shape[0])
+        col_start, col_stop, col_step = col.indices(self.shape[1])
+        row_range = xrange(row_start, row_stop, row_step)
+        col_range = xrange(col_start, col_stop, col_step)
+        shape = (len(row_range), len(col_range))
+        # Switch paths only when advantageous
+        # (count the iterations in the loops, adjust for complexity)
+        if len(self) >= 2 * shape[0] * shape[1]:
+            # O(nr*nc) path: loop over <row x col>
+            return self._get_columnXarray(row_range, col_range)
+        # O(nnz) path: loop over entries of self
+        newdok = dok_matrix(shape, dtype=self.dtype)
+        for key in iterkeys(self):
+            i, ri = divmod(int(key[0]) - row_start, row_step)
+            if ri != 0 or i < 0 or i >= shape[0]:
+                continue
+            j, rj = divmod(int(key[1]) - col_start, col_step)
+            if rj != 0 or j < 0 or j >= shape[1]:
+                continue
+            x = dict.__getitem__(self, key)
+            dict.__setitem__(newdok, (i, j), x)
+        return newdok
 
-        i, j = self._index_to_arrays(i, j)
+    def _get_intXarray(self, row, col):
+        return self._get_columnXarray([row], col)
 
-        if i.size == 0:
-            return dok_matrix(i.shape, dtype=self.dtype)
+    def _get_arrayXint(self, row, col):
+        return self._get_columnXarray(row, [col])
 
-        min_i = i.min()
-        if min_i < -self.shape[0] or i.max() >= self.shape[0]:
-            raise IndexError('Index (%d) out of range -%d to %d.' %
-                             (i.min(), self.shape[0], self.shape[0]-1))
-        if min_i < 0:
-            i = i.copy()
-            i[i < 0] += self.shape[0]
+    def _get_sliceXarray(self, row, col):
+        row = list(range(*row.indices(self.shape[0])))
+        return self._get_columnXarray(row, col)
 
-        min_j = j.min()
-        if min_j < -self.shape[1] or j.max() >= self.shape[1]:
-            raise IndexError('Index (%d) out of range -%d to %d.' %
-                             (j.min(), self.shape[1], self.shape[1]-1))
-        if min_j < 0:
-            j = j.copy()
-            j[j < 0] += self.shape[1]
+    def _get_arrayXslice(self, row, col):
+        col = list(range(*col.indices(self.shape[1])))
+        return self._get_columnXarray(row, col)
 
+    def _get_columnXarray(self, row, col):
+        # outer indexing
+        newdok = dok_matrix((len(row), len(col)), dtype=self.dtype)
+
+        for i, r in enumerate(row):
+            for j, c in enumerate(col):
+                v = dict.get(self, (r, c), 0)
+                if v:
+                    dict.__setitem__(newdok, (i, j), v)
+        return newdok
+
+    def _get_arrayXarray(self, row, col):
+        # inner indexing
+        i, j = map(np.atleast_2d, np.broadcast_arrays(row, col))
         newdok = dok_matrix(i.shape, dtype=self.dtype)
 
         for key in itertools.product(xrange(i.shape[0]), xrange(i.shape[1])):
-            v = dict.get(self, (i[key], j[key]), zero)
+            v = dict.get(self, (i[key], j[key]), 0)
             if v:
                 dict.__setitem__(newdok, key, v)
-
         return newdok
 
-    def _getitem_ranges(self, i_indices, j_indices, shape):
-        # performance golf: we don't want Numpy scalars here, they are slow
-        i_start, i_stop, i_stride = map(int, i_indices)
-        j_start, j_stop, j_stride = map(int, j_indices)
+    def _set_intXint(self, row, col, x):
+        key = (row, col)
+        if x:
+            dict.__setitem__(self, key, x)
+        elif dict.__contains__(self, key):
+            del self[key]
 
-        newdok = dok_matrix(shape, dtype=self.dtype)
+    def _set_arrayXarray(self, row, col, x):
+        row = list(map(int, row.ravel()))
+        col = list(map(int, col.ravel()))
+        x = x.ravel()
+        dict.update(self, izip(izip(row, col), x))
 
-        for (ii, jj) in iterkeys(self):
-            # ditto for numpy scalars
-            ii = int(ii)
-            jj = int(jj)
-            a, ra = divmod(ii - i_start, i_stride)
-            if a < 0 or a >= shape[0] or ra != 0:
-                continue
-            b, rb = divmod(jj - j_start, j_stride)
-            if b < 0 or b >= shape[1] or rb != 0:
-                continue
-            dict.__setitem__(newdok, (a, b),
-                             dict.__getitem__(self, (ii, jj)))
-        return newdok
-
-    def __setitem__(self, index, x):
-        if isinstance(index, tuple) and len(index) == 2:
-            # Integer index fast path
-            i, j = index
-            if (isintlike(i) and isintlike(j) and 0 <= i < self.shape[0]
-                    and 0 <= j < self.shape[1]):
-                v = np.asarray(x, dtype=self.dtype)
-                if v.ndim == 0 and v != 0:
-                    dict.__setitem__(self, (int(i), int(j)), v[()])
-                    return
-
-        i, j = self._unpack_index(index)
-        i, j = self._index_to_arrays(i, j)
-
-        if isspmatrix(x):
-            x = x.toarray()
-
-        # Make x and i into the same shape
-        x = np.asarray(x, dtype=self.dtype)
-        x, _ = np.broadcast_arrays(x, i)
-
-        if x.shape != i.shape:
-            raise ValueError("Shape mismatch in assignment.")
-
-        if np.size(x) == 0:
-            return
-
-        min_i = i.min()
-        if min_i < -self.shape[0] or i.max() >= self.shape[0]:
-            raise IndexError('Index (%d) out of range -%d to %d.' %
-                             (i.min(), self.shape[0], self.shape[0]-1))
-        if min_i < 0:
-            i = i.copy()
-            i[i < 0] += self.shape[0]
-
-        min_j = j.min()
-        if min_j < -self.shape[1] or j.max() >= self.shape[1]:
-            raise IndexError('Index (%d) out of range -%d to %d.' %
-                             (j.min(), self.shape[1], self.shape[1]-1))
-        if min_j < 0:
-            j = j.copy()
-            j[j < 0] += self.shape[1]
-
-        dict.update(self, izip(izip(i.flat, j.flat), x.flat))
-
-        if 0 in x:
-            zeroes = x == 0
-            for key in izip(i[zeroes].flat, j[zeroes].flat):
-                if dict.__getitem__(self, key) == 0:
-                    # may have been superseded by later update
-                    del self[key]
+        for i in np.nonzero(x == 0)[0]:
+            key = (row[i], col[i])
+            if dict.__getitem__(self, key) == 0:
+                # may have been superseded by later update
+                del self[key]
 
     def __add__(self, other):
         if isscalarlike(other):
@@ -452,18 +390,6 @@ class dok_matrix(spmatrix, IndexMixin, dict):
 
     copy.__doc__ = spmatrix.copy.__doc__
 
-    def getrow(self, i):
-        """Returns the i-th row as a (1 x n) DOK matrix."""
-        new = dok_matrix((1, self.shape[1]), dtype=self.dtype)
-        dict.update(new, (((0, j), self[i, j]) for j in xrange(self.shape[1])))
-        return new
-
-    def getcol(self, j):
-        """Returns the j-th column as a (m x 1) DOK matrix."""
-        new = dok_matrix((self.shape[0], 1), dtype=self.dtype)
-        dict.update(new, (((i, 0), self[i, j]) for i in xrange(self.shape[0])))
-        return new
-
     def tocoo(self, copy=False):
         from .coo import coo_matrix
         if self.nnz == 0:
@@ -529,10 +455,3 @@ def isspmatrix_dok(x):
     False
     """
     return isinstance(x, dok_matrix)
-
-
-def _prod(x):
-    """Product of a list of numbers; ~40x faster vs np.prod for Python tuples"""
-    if len(x) == 0:
-        return 1
-    return functools.reduce(operator.mul, x)
