@@ -1,4 +1,4 @@
-"""LInked List sparse matrix class
+"""List of Lists sparse matrix class
 """
 
 from __future__ import division, print_function, absolute_import
@@ -13,7 +13,7 @@ import numpy as np
 
 from scipy._lib.six import xrange, zip
 from .base import spmatrix, isspmatrix
-from ._index import IndexMixin, INT_TYPES
+from ._index import IndexMixin, INT_TYPES, _broadcast_arrays
 from .sputils import (getdtype, isshape, isscalarlike, upcast_scalar,
                       get_index_dtype, check_shape, check_reshape_kwargs,
                       asmatrix)
@@ -21,7 +21,7 @@ from . import _csparsetools
 
 
 class lil_matrix(spmatrix, IndexMixin):
-    """Row-based linked list sparse matrix
+    """Row-based list of lists sparse matrix
 
     This is a structure for constructing sparse matrices incrementally.
     Note that inserting a single item can take linear time in the worst case;
@@ -48,7 +48,7 @@ class lil_matrix(spmatrix, IndexMixin):
     ndim : int
         Number of dimensions (this is always 2)
     nnz
-        Number of nonzero elements
+        Number of stored values, including explicit zeros
     data
         LIL format data array of the matrix
     rows
@@ -225,16 +225,6 @@ class lil_matrix(spmatrix, IndexMixin):
             raise IndexError('Index dimension must be <= 2')
         return x
 
-    def __getitem__(self, key):
-        # Fast path for simple (int, int) indexing.
-        if (isinstance(key, tuple) and len(key) == 2 and
-                isinstance(key[0], INT_TYPES) and
-                isinstance(key[1], INT_TYPES)):
-            # lil_get1 handles validation for us.
-            return self._get_intXint(*key)
-        # Everything else takes the normal path.
-        return IndexMixin.__getitem__(self, key)
-
     def _get_intXint(self, row, col):
         v = _csparsetools.lil_get1(self.shape[0], self.shape[1], self.rows,
                                    self.data, row, col)
@@ -267,7 +257,7 @@ class lil_matrix(spmatrix, IndexMixin):
 
     def _get_columnXarray(self, row, col):
         # outer indexing
-        row, col = np.broadcast_arrays(row[:,None], col)
+        row, col = _broadcast_arrays(row[:,None], col)
         return self._get_arrayXarray(row, col)
 
     def _get_arrayXarray(self, row, col):
@@ -330,7 +320,7 @@ class lil_matrix(spmatrix, IndexMixin):
             return
         # Fall back to densifying x
         x = np.asarray(x.toarray(), dtype=self.dtype)
-        x, _ = np.broadcast_arrays(x, row)
+        x, _ = _broadcast_arrays(x, row)
         self._set_arrayXarray(row, col, x)
 
     def __setitem__(self, key, x):
@@ -460,13 +450,36 @@ class lil_matrix(spmatrix, IndexMixin):
     tolil.__doc__ = spmatrix.tolil.__doc__
 
     def tocsr(self, copy=False):
-        lst = [len(x) for x in self.rows]
-        idx_dtype = get_index_dtype(maxval=max(self.shape[1], sum(lst)))
-
-        indptr = np.cumsum([0] + lst, dtype=idx_dtype)
-        indices = np.array([x for y in self.rows for x in y], dtype=idx_dtype)
-        data = np.array([x for y in self.data for x in y], dtype=self.dtype)
-
+        # construct indptr array
+        M, N = self.shape
+        lengths = np.fromiter(map(len, self.rows),
+                              dtype=get_index_dtype(maxval=N), count=M)
+        nnz = lengths.sum()
+        idx_dtype = get_index_dtype(maxval=max(N, nnz))
+        indptr = np.empty(M + 1, dtype=idx_dtype)
+        indptr[0] = 0
+        np.cumsum(lengths, dtype=idx_dtype, out=indptr[1:])
+        # construct indices and data array
+        # using faster construction approach depending on density
+        # see https://github.com/scipy/scipy/pull/10939 for details
+        if M == 0:
+            indices = np.empty(0, dtype=idx_dtype)
+            data = np.empty(0, dtype=self.dtype)
+        elif nnz / M > 30:
+            indices = np.empty(nnz, dtype=idx_dtype)
+            data = np.empty(nnz, dtype=self.dtype)
+            start = 0
+            for i, stop in enumerate(indptr[1:]):
+                if stop > start:
+                    indices[start:stop] = self.rows[i]
+                    data[start:stop] = self.data[i]
+                    start = stop
+        else:
+            indices = np.fromiter((x for y in self.rows for x in y),
+                                  dtype=idx_dtype, count=nnz)
+            data = np.fromiter((x for y in self.data for x in y),
+                               dtype=self.dtype, count=nnz)
+        # init csr matrix
         from .csr import csr_matrix
         return csr_matrix((data, indices, indptr), shape=self.shape)
 
