@@ -9,6 +9,7 @@ import pickle
 from numpy.testing import (assert_allclose, assert_almost_equal,
                            assert_array_almost_equal, assert_equal,
                            assert_array_less, assert_)
+import pytest
 from pytest import raises as assert_raises
 
 from .test_continuous_basic import check_distribution_rvs
@@ -17,7 +18,7 @@ import numpy
 import numpy as np
 
 import scipy.linalg
-from scipy.stats._multivariate import _PSD, _lnB
+from scipy.stats._multivariate import _PSD, _lnB, _cho_inv_batch
 from scipy.stats import multivariate_normal
 from scipy.stats import matrix_normal
 from scipy.stats import special_ortho_group, ortho_group
@@ -30,6 +31,7 @@ from scipy.stats import ks_2samp, kstest
 from scipy.stats import binom
 
 from scipy.integrate import romb
+from scipy.special import multigammaln
 
 from .common_tests import check_random_state_property
 
@@ -281,7 +283,7 @@ class TestMultivariateNormal(object):
         x = np.random.randn(n, n)
         cov = np.dot(x, x.T)
         s, u = scipy.linalg.eigh(cov)
-        s = 0.5 * np.ones(n)
+        s = np.full(n, 0.5)
         s[0] = 1.0
         s[-1] = 1e-7
         cov = np.dot(u, np.dot(np.diag(s), u.T))
@@ -459,9 +461,9 @@ class TestMatrixNormal(object):
         # Check that bad inputs raise errors
         num_rows = 4
         num_cols = 3
-        M = 0.3 * np.ones((num_rows,num_cols))
-        U = 0.5 * np.identity(num_rows) + 0.5 * np.ones((num_rows, num_rows))
-        V = 0.7 * np.identity(num_cols) + 0.3 * np.ones((num_cols, num_cols))
+        M = np.full((num_rows,num_cols), 0.3)
+        U = 0.5 * np.identity(num_rows) + np.full((num_rows, num_rows), 0.5)
+        V = 0.7 * np.identity(num_cols) + np.full((num_cols, num_cols), 0.3)
 
         # Incorrect dimensions
         assert_raises(ValueError, matrix_normal, np.zeros((5,4,3)))
@@ -480,9 +482,9 @@ class TestMatrixNormal(object):
         # Check that default argument handling works
         num_rows = 4
         num_cols = 3
-        M = 0.3 * np.ones((num_rows,num_cols))
-        U = 0.5 * np.identity(num_rows) + 0.5 * np.ones((num_rows, num_rows))
-        V = 0.7 * np.identity(num_cols) + 0.3 * np.ones((num_cols, num_cols))
+        M = np.full((num_rows,num_cols), 0.3)
+        U = 0.5 * np.identity(num_rows) + np.full((num_rows, num_rows), 0.5)
+        V = 0.7 * np.identity(num_cols) + np.full((num_cols, num_cols), 0.3)
         Z = np.zeros((num_rows, num_cols))
         Zr = np.zeros((num_rows, 1))
         Zc = np.zeros((1, num_cols))
@@ -519,10 +521,10 @@ class TestMatrixNormal(object):
         # Check that covariance can be specified with scalar or vector
         num_rows = 4
         num_cols = 3
-        M = 0.3 * np.ones((num_rows,num_cols))
-        Uv = 0.2*np.ones(num_rows)
+        M = np.full((num_rows, num_cols), 0.3)
+        Uv = np.full(num_rows, 0.2)
         Us = 0.2
-        Vv = 0.1*np.ones(num_cols)
+        Vv = np.full(num_cols, 0.1)
         Vs = 0.1
 
         Ir = np.identity(num_rows)
@@ -540,9 +542,9 @@ class TestMatrixNormal(object):
     def test_frozen_matrix_normal(self):
         for i in range(1,5):
             for j in range(1,5):
-                M = 0.3 * np.ones((i,j))
-                U = 0.5 * np.identity(i) + 0.5 * np.ones((i,i))
-                V = 0.7 * np.identity(j) + 0.3 * np.ones((j,j))
+                M = np.full((i,j), 0.3)
+                U = 0.5 * np.identity(i) + np.full((i,i), 0.5)
+                V = 0.7 * np.identity(j) + np.full((j,j), 0.3)
 
                 frozen = matrix_normal(mean=M, rowcov=U, colcov=V)
 
@@ -566,9 +568,9 @@ class TestMatrixNormal(object):
         # treating as a multivariate normal.
         for i in range(1,5):
             for j in range(1,5):
-                M = 0.3 * np.ones((i,j))
-                U = 0.5 * np.identity(i) + 0.5 * np.ones((i,i))
-                V = 0.7 * np.identity(j) + 0.3 * np.ones((j,j))
+                M = np.full((i,j), 0.3)
+                U = 0.5 * np.identity(i) + np.full((i,i), 0.5)
+                V = 0.7 * np.identity(j) + np.full((j,j), 0.3)
 
                 frozen = matrix_normal(mean=M, rowcov=U, colcov=V)
                 X = frozen.rvs(random_state=1234)
@@ -588,9 +590,9 @@ class TestMatrixNormal(object):
         # Check array of inputs has the same output as the separate entries.
         num_rows = 4
         num_cols = 3
-        M = 0.3 * np.ones((num_rows,num_cols))
-        U = 0.5 * np.identity(num_rows) + 0.5 * np.ones((num_rows, num_rows))
-        V = 0.7 * np.identity(num_cols) + 0.3 * np.ones((num_cols, num_cols))
+        M = np.full((num_rows,num_cols), 0.3)
+        U = 0.5 * np.identity(num_rows) + np.full((num_rows, num_rows), 0.5)
+        V = 0.7 * np.identity(num_cols) + np.full((num_cols, num_cols), 0.3)
         N = 10
 
         frozen = matrix_normal(mean=M, rowcov=U, colcov=V)
@@ -611,9 +613,9 @@ class TestMatrixNormal(object):
         # Check that the sample moments match the parameters
         num_rows = 4
         num_cols = 3
-        M = 0.3 * np.ones((num_rows,num_cols))
-        U = 0.5 * np.identity(num_rows) + 0.5 * np.ones((num_rows, num_rows))
-        V = 0.7 * np.identity(num_cols) + 0.3 * np.ones((num_cols, num_cols))
+        M = np.full((num_rows,num_cols), 0.3)
+        U = 0.5 * np.identity(num_rows) + np.full((num_rows, num_rows), 0.5)
+        V = 0.7 * np.identity(num_cols) + np.full((num_cols, num_cols), 0.3)
         N = 1000
 
         frozen = matrix_normal(mean=M, rowcov=U, colcov=V)
@@ -680,6 +682,15 @@ class TestDirichlet(object):
     def test_data_with_zeros(self):
         alpha = np.array([1.0, 2.0, 3.0, 4.0])
         x = np.array([0.1, 0.0, 0.2, 0.7])
+        dirichlet.pdf(x, alpha)
+        dirichlet.logpdf(x, alpha)
+        alpha = np.array([1.0, 1.0, 1.0, 1.0])
+        assert_almost_equal(dirichlet.pdf(x, alpha), 6)
+        assert_almost_equal(dirichlet.logpdf(x, alpha), np.log(6))
+
+    def test_data_with_zeros_and_small_alpha(self):
+        alpha = np.array([1.0, 0.5, 3.0, 4.0])
+        x = np.array([0.1, 0.0, 0.2, 0.7])
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
@@ -697,37 +708,37 @@ class TestDirichlet(object):
 
     def test_data_too_deep_c(self):
         alpha = np.array([1.0, 2.0, 3.0])
-        x = np.ones((2, 7, 7)) / 14
+        x = np.full((2, 7, 7), 1 / 14)
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
     def test_alpha_too_deep(self):
         alpha = np.array([[1.0, 2.0], [3.0, 4.0]])
-        x = np.ones((2, 2, 7)) / 4
+        x = np.full((2, 2, 7), 1 / 4)
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
     def test_alpha_correct_depth(self):
         alpha = np.array([1.0, 2.0, 3.0])
-        x = np.ones((3, 7)) / 3
+        x = np.full((3, 7), 1 / 3)
         dirichlet.pdf(x, alpha)
         dirichlet.logpdf(x, alpha)
 
     def test_non_simplex_data(self):
         alpha = np.array([1.0, 2.0, 3.0])
-        x = np.ones((3, 7)) / 2
+        x = np.full((3, 7), 1 / 2)
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
     def test_data_vector_too_short(self):
         alpha = np.array([1.0, 2.0, 3.0, 4.0])
-        x = np.ones((2, 7)) / 2
+        x = np.full((2, 7), 1 / 2)
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
     def test_data_vector_too_long(self):
         alpha = np.array([1.0, 2.0, 3.0, 4.0])
-        x = np.ones((5, 7)) / 5
+        x = np.full((5, 7), 1 / 5)
         assert_raises(ValueError, dirichlet.pdf, x, alpha)
         assert_raises(ValueError, dirichlet.logpdf, x, alpha)
 
@@ -1083,6 +1094,9 @@ class TestMultinomial(object):
         vals4 = multinomial.pmf([1,2], 4, (.3, .7))
         assert_allclose(vals4, 0, rtol=1e-8)
 
+        vals5 = multinomial.pmf([3, 3, 0], 6, [2/3.0, 1/3.0, 0])
+        assert_allclose(vals5, 0.219478737997, rtol=1e-8)
+
     def test_pmf_broadcasting(self):
         vals0 = multinomial.pmf([1, 2], 3, [[.1, .9], [.2, .8]])
         assert_allclose(vals0, [.243, .384], rtol=1e-8)
@@ -1283,14 +1297,55 @@ class TestInvwishart(object):
         assert_allclose(iw_rvs, manual_iw_rvs)
         assert_allclose(frozen_iw_rvs, manual_iw_rvs)
 
+    def test_cho_inv_batch(self):
+        """Regression test for gh-8844."""
+        a0 = np.array([[2, 1, 0, 0.5],
+                       [1, 2, 0.5, 0.5],
+                       [0, 0.5, 3, 1],
+                       [0.5, 0.5, 1, 2]])
+        a1 = np.array([[2, -1, 0, 0.5],
+                       [-1, 2, 0.5, 0.5],
+                       [0, 0.5, 3, 1],
+                       [0.5, 0.5, 1, 4]])
+        a = np.array([a0, a1])
+        ainv = a.copy()
+        _cho_inv_batch(ainv)
+        ident = np.eye(4)
+        assert_allclose(a[0].dot(ainv[0]), ident, atol=1e-15)
+        assert_allclose(a[1].dot(ainv[1]), ident, atol=1e-15)
+
+    def test_logpdf_4x4(self):
+        """Regression test for gh-8844."""
+        X = np.array([[2, 1, 0, 0.5],
+                      [1, 2, 0.5, 0.5],
+                      [0, 0.5, 3, 1],
+                      [0.5, 0.5, 1, 2]])
+        Psi = np.array([[9, 7, 3, 1],
+                        [7, 9, 5, 1],
+                        [3, 5, 8, 2],
+                        [1, 1, 2, 9]])
+        nu = 6
+        prob = invwishart.logpdf(X, nu, Psi)
+        # Explicit calculation from the formula on wikipedia.
+        p = X.shape[0]
+        sig, logdetX = np.linalg.slogdet(X)
+        sig, logdetPsi = np.linalg.slogdet(Psi)
+        M = np.linalg.solve(X, Psi)
+        expected = ((nu/2)*logdetPsi
+                    - (nu*p/2)*np.log(2)
+                    - multigammaln(nu/2, p)
+                    - (nu + p + 1)/2*logdetX
+                    - 0.5*M.trace())
+        assert_allclose(prob, expected)
+
 
 class TestSpecialOrthoGroup(object):
     def test_reproducibility(self):
         np.random.seed(514)
         x = special_ortho_group.rvs(3)
-        expected = np.array([[0.99394515, -0.04527879, 0.10011432],
-                             [-0.04821555, 0.63900322, 0.76769144],
-                             [-0.09873351, -0.76787024, 0.63295101]])
+        expected = np.array([[-0.99394515, -0.04527879, 0.10011432],
+                             [0.04821555, -0.99846897, 0.02711042],
+                             [0.09873351, 0.03177334, 0.99460653]])
         assert_array_almost_equal(x, expected)
 
         random_state = np.random.RandomState(seed=514)
@@ -1334,7 +1389,7 @@ class TestSpecialOrthoGroup(object):
         # Generate samples
         dim = 5
         samples = 1000  # Not too many, or the test takes too long
-        ks_prob = 0.39  # ...so don't expect much precision
+        ks_prob = .05
         np.random.seed(514)
         xs = special_ortho_group.rvs(dim, size=samples)
 
@@ -1355,16 +1410,16 @@ class TestSpecialOrthoGroup(object):
 
 class TestOrthoGroup(object):
     def test_reproducibility(self):
-        np.random.seed(514)
+        np.random.seed(515)
         x = ortho_group.rvs(3)
-        x2 = ortho_group.rvs(3, random_state=514)
+        x2 = ortho_group.rvs(3, random_state=515)
         # Note this matrix has det -1, distinguishing O(N) from SO(N)
-        expected = np.array([[0.993945, -0.045279, 0.100114],
-                             [-0.048216, -0.998469, 0.02711],
-                             [-0.098734, 0.031773, 0.994607]])
+        assert_almost_equal(np.linalg.det(x), -1)
+        expected = np.array([[0.94449759, -0.21678569, -0.24683651],
+                             [-0.13147569, -0.93800245, 0.3207266],
+                             [0.30106219, 0.27047251, 0.9144431]])
         assert_array_almost_equal(x, expected)
         assert_array_almost_equal(x2, expected)
-        assert_almost_equal(np.linalg.det(x), -1)
 
     def test_invalid_dim(self):
         assert_raises(ValueError, ortho_group.rvs, None)
@@ -1373,18 +1428,25 @@ class TestOrthoGroup(object):
         assert_raises(ValueError, ortho_group.rvs, 2.5)
 
     def test_det_and_ortho(self):
-        xs = [ortho_group.rvs(dim)
-              for dim in range(2,12)
-              for i in range(3)]
+        xs = [[ortho_group.rvs(dim)
+               for i in range(10)]
+              for dim in range(2,12)]
 
-        # Test that determinants are always +1
-        dets = [np.fabs(np.linalg.det(x)) for x in xs]
-        assert_allclose(dets, [1.]*30, rtol=1e-13)
+        # Test that abs determinants are always +1
+        dets = np.array([[np.linalg.det(x) for x in xx] for xx in xs])
+        assert_allclose(np.fabs(dets), np.ones(dets.shape), rtol=1e-13)
+
+        # Test that we get both positive and negative determinants
+        # Check that we have at least one and less than 10 negative dets in a sample of 10. The rest are positive by the previous test.
+        # Test each dimension separately
+        assert_array_less([0]*10, [np.nonzero(d < 0)[0].shape[0] for d in dets])
+        assert_array_less([np.nonzero(d < 0)[0].shape[0] for d in dets], [10]*10)
 
         # Test that these are orthogonal matrices
-        for x in xs:
-            assert_array_almost_equal(np.dot(x, x.T),
-                                      np.eye(x.shape[0]))
+        for xx in xs:
+            for x in xx:
+                assert_array_almost_equal(np.dot(x, x.T),
+                                          np.eye(x.shape[0]))
 
     def test_haar(self):
         # Test that the distribution is constant under rotation
@@ -1394,7 +1456,7 @@ class TestOrthoGroup(object):
         # Generate samples
         dim = 5
         samples = 1000  # Not too many, or the test takes too long
-        ks_prob = 0.39  # ...so don't expect much precision
+        ks_prob = .05
         np.random.seed(518)  # Note that the test is sensitive to seed too
         xs = ortho_group.rvs(dim, size=samples)
 
@@ -1413,6 +1475,32 @@ class TestOrthoGroup(object):
         ks_tests = [ks_2samp(proj[p0], proj[p1])[1] for (p0, p1) in pairs]
         assert_array_less([ks_prob]*len(pairs), ks_tests)
 
+    @pytest.mark.slow
+    def test_pairwise_distances(self):
+        # Test that the distribution of pairwise distances is close to correct.
+        np.random.seed(514)
+
+        def random_ortho(dim):
+            u, _s, v = np.linalg.svd(np.random.normal(size=(dim, dim)))
+            return np.dot(u, v)
+
+        for dim in range(2, 6):
+            def generate_test_statistics(rvs, N=1000, eps=1e-10):
+                stats = np.array([
+                    np.sum((rvs(dim=dim) - rvs(dim=dim))**2)
+                    for _ in range(N)
+                ])
+                # Add a bit of noise to account for numeric accuracy.
+                stats += np.random.uniform(-eps, eps, size=stats.shape)
+                return stats
+
+            expected = generate_test_statistics(random_ortho)
+            actual = generate_test_statistics(scipy.stats.ortho_group.rvs)
+
+            _D, p = scipy.stats.ks_2samp(expected, actual)
+
+            assert_array_less(.05, p)
+
 class TestRandomCorrelation(object):
     def test_reproducibility(self):
         np.random.seed(514)
@@ -1436,7 +1524,7 @@ class TestRandomCorrelation(object):
         assert_raises(ValueError, random_correlation.rvs, [1, 2, .1])
 
     def test_definition(self):
-        # Test the defintion of a correlation matrix in several dimensions:
+        # Test the definition of a correlation matrix in several dimensions:
         #
         # 1. Det is product of eigenvalues (and positive by construction
         #    in examples)
@@ -1544,7 +1632,7 @@ class TestUnitaryGroup(object):
 
         # The angles "x" of the eigenvalues should be uniformly distributed
         # Overall this seems to be a necessary but weak test of the distribution.
-        eigs = np.vstack(scipy.linalg.eigvals(x) for x in xs)
+        eigs = np.vstack([scipy.linalg.eigvals(x) for x in xs])
         x = np.arctan2(eigs.imag, eigs.real)
         res = kstest(x.ravel(), uniform(-np.pi, 2*np.pi).cdf)
         assert_(res.pvalue > 0.05)
